package gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import java.util.ResourceBundle;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import model.Category;
import model.ModelImplementation;
import model.ModelInterface;
import model.Note;

import controller.CreateAction;
import controller.NewNoteAction;
import controller.ActionMap;
import controller.ResetAction;

public class CreateNote extends TabContent implements CreateSaveInterface{
	
	String author = System.getProperty("user.name");
	
	// Components
	JLabel lKeyword = new JLabel(rb.getString("keyword"));
	JLabel lNote = new JLabel(rb.getString("note"));
	
	// TODO later implement with setter-methods
	public JTextField tfKeyword = new JTextField(20);

	JButton createButton;
	JButton resetButton;
	
	public CreateNote(MainGuiFrame refMainGui){ 
		super(refMainGui);
		
		createCategoryPanel();
		
		setTabName(rb.getString("createNote"));
		
		setComponentLayout();
		
		createButtonsForCreateNote();
		
		tfTitle.getDocument().addDocumentListener(new ContentTitleListener());
		
		mainGui.addNewTab(this);
	}
	
	// Checks if Titlefield tfTitle has some content
	private class ContentTitleListener implements DocumentListener{

		public void changedUpdate(DocumentEvent e) {
			titleContentCheck(e);
		}

		public void insertUpdate(DocumentEvent e) {
			titleContentCheck(e);
		}

		public void removeUpdate(DocumentEvent e) {
			titleContentCheck(e);
		}
		
		public void titleContentCheck(DocumentEvent ev){
			createButton.setEnabled(tfTitle.getText().length() > 0);
		}
	}

	public void createButtonsForCreateNote() {
		CreateAction ca = createAndRegisterCreateAction();
		
		createAndRegisterNewNoteAction();
		
		ResetAction ra = createAndRegisterResetAction();
		
		resetButton = new JButton(ra);
		createButton = new JButton(ca);
		
		addButton(createButton);
		addButton(resetButton);
		createButton.setEnabled(false);
	}

	private ResetAction createAndRegisterResetAction() {
		Action resetAction = ActionMap.getAction("reset");
		ResetAction ra = (ResetAction) resetAction;
		ra.setCreateNote(this);
		return ra;
	}

	private void createAndRegisterNewNoteAction() {
		Action newNoteAction = ActionMap.getAction("newNote");
		NewNoteAction nna = (NewNoteAction) newNoteAction;
		nna.setMainGui(mainGui);
		nna.setCreateNote(this);
	}

	public CreateAction createAndRegisterCreateAction() {
		Action createAction = ActionMap.getAction("create");
		CreateAction ca = (CreateAction) createAction;
		ca.setCreateSaveInterface(this);
		return ca;
	}

	public void setComponentLayout() {
		constraints.anchor = GridBagConstraints.SOUTHWEST;
		setComponent(constraints, 0, 0, 1, 1, lTitle);		
		setComponent(constraints, 1, 0, 1, 1, lKeyword);		
		setComponent(constraints, 0, 2, 1, 2, lNote);
		setComponent(constraints, 2, 0, 1, 1, lCategory);

		constraints.anchor = GridBagConstraints.NORTHWEST;
		setComponent(constraints, 0, 1, 1, 1, tfTitle);		
		setComponent(constraints, 1, 1, 1, 1, tfKeyword);
		
		constraints.fill = GridBagConstraints.BOTH;
		setComponent(constraints, 0, 3, 1, 2, new JScrollPane(taNote));
		setComponent(constraints, 2, 1, 3, 1, spCategory);
	}
	
//	public static void main(String[] args) {
//		JFrame jf = new JFrame();
//		jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//		jf.add(new CreateNote(new MainGuiFrame(), note), BorderLayout.CENTER);
//		jf.setSize(400, 200);
//		jf.setVisible(true);
//	}

	public String getNote() {
		return taNote.getText();
	}

	public String getTitle() {
		return tfTitle.getText();
	}
	
	public String getKeyword(){
		return tfKeyword.getText();
	}

	public Note getNoteVersion() {
		return null;
	}
	
	public Object getCategory(){
		return null;
	}

//	@Override
	public int smallLines() {
		return 3;
	}

	public String getAuthor() {
		return author;
	}

	public Date getDateFrom() {
		// TODO Auto-generated method stub
		return null;
	}

	public Date getDateTo() {
		// TODO Auto-generated method stub
		return null;
	}
	
	public void resetFields(){
		tfTitle.setText("");
		tfKeyword.setText("");
		taNote.setText("");
	}
}
