package gui;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.util.ResourceBundle;

import controller.MenuSearchAction;
import controller.ActionMap;
import controller.SearchAction;

import model.Category;
import model.ModelImplementation;
import model.ModelInterface;
import model.Note;

public class SearchNote extends TabContent implements CreateSaveInterface {
	
	private static final String DATE_DEFAULT = "yyyy-MM-dd";

	JLabel lKeyword = new JLabel(rb.getString("keywords"));
	JLabel lAuthor = new JLabel(rb.getString("author"));
	JLabel lDateFrom = new JLabel(rb.getString("datefrom"));
	JLabel lDateTo = new JLabel(rb.getString("dateto"));
	JLabel lNote = new JLabel(rb.getString("fulltext"));

	JTextField tfKeyword = new JTextField(20);
	JTextField tfAuthor = new JTextField(20);
	JTextField tfDateFrom = new JTextField(DATE_DEFAULT, 10);
	JTextField tfDateTo = new JTextField(DATE_DEFAULT, 10);

	JButton searchButton;

	public SearchNote(MainGuiFrame refMainGui) {
		
		super(refMainGui);
		
		createCategoryPanel();
		
		setTabName(rb.getString("searchNote"));
		
		setComponentLayout();
	
		CreateButtonsForSearchNote();
		
		tfDateFrom.getDocument().addDocumentListener(new MyDocumentListener(tfDateFrom));
		tfDateTo.getDocument().addDocumentListener(new MyDocumentListener(tfDateTo));

		mainGui.addNewTab(this);
	}

	private void CreateButtonsForSearchNote() {
		SearchAction sa = createAndRegisterSearchAction();
		createAndRegisterMenuSearchAction();
		
		searchButton = new JButton(sa);

		addButton(searchButton);
	}

	public void createAndRegisterMenuSearchAction() {
		Action menuSearchAction = ActionMap.getAction("menuSearch");
		MenuSearchAction msa = (MenuSearchAction) menuSearchAction;
		msa.setMainGuiReference(mainGui);
		msa.setSearchNote(this);
	}

	public SearchAction createAndRegisterSearchAction() {
		Action searchAction = ActionMap.getAction("search");
		SearchAction sa = (SearchAction) searchAction;
		sa.setCreateSaveInterface(this);
		sa.setMainGuiReference(mainGui);
		return sa;
	}

	public void setComponentLayout() {
		constraints.anchor = GridBagConstraints.SOUTHWEST;
		setComponent(constraints, 0, 0, 1, 1, lTitle);
		setComponent(constraints, 1, 0, 1, 1, lKeyword);
		setComponent(constraints, 2, 0, 1, 1, lCategory);
		setComponent(constraints, 0, 1, 1, 1, tfTitle);
		setComponent(constraints, 1, 1, 1, 1, tfKeyword);
		setComponent(constraints, 0, 2, 1, 1, lAuthor);
		setComponent(constraints, 0, 3, 1, 1, tfAuthor);
		setComponent(constraints, 0, 4, 1, 1, lDateFrom);
		setComponent(constraints, 1, 4, 1, 1, lDateTo);
		setComponent(constraints, 0, 5, 1, 1, tfDateFrom);
		setComponent(constraints, 1, 5, 1, 1, tfDateTo);
		setComponent(constraints, 0, 6, 1, 2, lNote);

		constraints.fill = GridBagConstraints.BOTH;
		setComponent(constraints, 0, 7, 1, 2, taNote);
		setComponent(constraints, 2, 1, 7, 1, spCategory);
	}

	class MyDocumentListener implements DocumentListener{
		JTextField dateField;
		
		public MyDocumentListener(JTextField _dateField){		
			dateField = _dateField;
		}
		
		public void changedUpdate(DocumentEvent e) {
			titleContentCheck(e);
		}

		public void insertUpdate(DocumentEvent e) {
			titleContentCheck(e);
		}

		public void removeUpdate(DocumentEvent e) {
			titleContentCheck(e);
		}
		
		public void titleContentCheck(DocumentEvent ev){
			//TODO SearchNote DatumsFormat: http://www.bastie.de/index.html?/java/howto/swing/text/javainputvalidationcheckverifier.html
			
			String dateText = dateField.getText();	
			
			boolean isDefault = dateText.equalsIgnoreCase(DATE_DEFAULT);
			boolean isEmpty = dateText.equalsIgnoreCase("");
			boolean isOk;
			
			isOk = (parseDate(dateText) != null);
			
			if(isDefault || isEmpty || isOk){
				dateField.setBackground(Color.WHITE);
				searchButton.setEnabled(true);
			} else {
				dateField.setBackground(Color.RED);
				searchButton.setEnabled(false);
			}
		}
	}
	
	public String getTitle() {
		return tfTitle.getText();
	}

	public String getKeyword() {
		return tfKeyword.getText();
	}

	public String getNote() {
		return taNote.getText();
	}

	public Object getCategory() {
		//TODO SearchNote getCategory
		return null;
	}

	public Note getNoteVersion() {
		//TODO SearchNote getNoteVersion
		return null;
	}

	public String getAuthor() {
		return tfAuthor.getText();
	}
	
	public Date getDateFrom() {
		String dateFrom = tfDateFrom.getText();
		return getDates(dateFrom);
	}

	public Date getDateTo() {
		String dateTo = tfDateTo.getText();
		return getDates(dateTo);
	}
	
	public Date getDates(String date){
		if(DATE_DEFAULT.equalsIgnoreCase(date)){
			return null;
		}

		return parseDate(date);
	}
	
	public Date parseDate(String date){
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_DEFAULT);
		Date dateD = null;
		
		try{
			dateD = sdf.parse(date);	
		} catch (ParseException ex){
		}
		return dateD;
	}
	
	public int smallLines() {
		return 7;
	}
}
